@@ -10,9 +10,11 @@ import (
 	"github.com/kiegroup/kie-cloud-operator/pkg/controller/kieapp/constants"
 	oappsv1 "github.com/openshift/api/apps/v1"
 	buildv1 "github.com/openshift/api/build/v1"
+	consolev1 "github.com/openshift/api/console/v1"
 	oimagev1 "github.com/openshift/api/image/v1"
 	routev1 "github.com/openshift/api/route/v1"
 	csvv1 "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"golang.org/x/mod/semver"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
@@ -113,44 +115,54 @@ func GetDeployment(operatorName, repository, context, imageName, tag, imagePullP
 	}
 	sort.Sort(sort.Reverse(sort.StringSlice(constants.SupportedVersions)))
 	for _, imageVersion := range constants.SupportedVersions {
-		if imageVersion >= "7.7.0" {
-			for _, i := range constants.Images {
-				env := corev1.EnvVar{
-					Name:  i.Var + imageVersion,
-					Value: i.Registry + ":" + imageVersion,
-				}
-				deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, env)
-			}
-			if versionConstants, found := constants.VersionConstants[imageVersion]; found {
-				deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
-					Name:  constants.OseCliVar + imageVersion,
-					Value: versionConstants.OseCliImageURL,
-				})
-				deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
-					Name:  constants.MySQLVar + imageVersion,
-					Value: versionConstants.MySQLImageURL,
-				})
-				deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
-					Name:  constants.PostgreSQLVar + imageVersion,
-					Value: versionConstants.PostgreSQLImageURL,
-				})
-				deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
-					Name:  constants.DatagridVar + imageVersion,
-					Value: versionConstants.DatagridImageURL,
-				})
-				deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
-					Name:  constants.BrokerVar + imageVersion,
-					Value: versionConstants.BrokerImageURL,
-				})
+		for _, i := range constants.Images {
+			if i.Var == constants.PamProcessMigrationVar && semver.Compare(semver.MajorMinor("v"+imageVersion), "v7.8") < 0 {
+				continue
 			}
+			deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+				Name:  i.Var + imageVersion,
+				Value: i.Registry + ":" + imageVersion,
+			})
+		}
+		if versionConstants, found := constants.VersionConstants[imageVersion]; found {
+			deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+				Name:  constants.OseCliVar + imageVersion,
+				Value: versionConstants.OseCliImageURL,
+			})
+			deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+				Name:  constants.MySQLVar + imageVersion,
+				Value: versionConstants.MySQLImageURL,
+			})
+			deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+				Name:  constants.PostgreSQLVar + imageVersion,
+				Value: versionConstants.PostgreSQLImageURL,
+			})
+			deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+				Name:  constants.DatagridVar + imageVersion,
+				Value: versionConstants.DatagridImageURL,
+			})
+			deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+				Name:  constants.BrokerVar + imageVersion,
+				Value: versionConstants.BrokerImageURL,
+			})
 		}
 	}
-	// add oauth-proxy image reference
+	// add oauth-proxy image references
 	deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
-		Name:  constants.OauthVar,
-		Value: constants.OauthImageURL,
+		Name:  constants.OauthVar + "LATEST",
+		Value: constants.Oauth4ImageLatestURL,
+	})
+	sort.Sort(sort.Reverse(sort.StringSlice(constants.Ocp4Versions)))
+	for _, ocpVersion := range constants.Ocp4Versions {
+		deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+			Name:  constants.OauthVar + ocpVersion,
+			Value: constants.Oauth4ImageURL + ":v" + ocpVersion,
+		})
+	}
+	deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{
+		Name:  constants.OauthVar + "3",
+		Value: constants.Oauth3ImageLatestURL,
 	})
-	//
 
 	return deployment
 }
@@ -168,16 +180,86 @@ func GetRole(operatorName string) *rbacv1.Role {
 			{
 				APIGroups: []string{
 					"",
+				},
+				Resources: []string{
+					"configmaps",
+					"pods",
+					"services",
+					"services/finalizers",
+					"serviceaccounts",
+					"persistentvolumeclaims",
+					"secrets",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					appsv1.SchemeGroupVersion.Group,
+				},
+				Resources: []string{
+					"deployments",
+					"deployments/finalizers",
+					"replicasets",
+					"statefulsets",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					oappsv1.SchemeGroupVersion.Group,
+				},
+				Resources: []string{
+					"deploymentconfigs",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					rbacv1.SchemeGroupVersion.Group,
+				},
+				Resources: []string{
+					"rolebindings",
+					"roles",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					routev1.SchemeGroupVersion.Group,
+				},
+				Resources: []string{
+					"routes",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					buildv1.SchemeGroupVersion.Group,
+				},
+				Resources: []string{
+					"buildconfigs",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					oimagev1.SchemeGroupVersion.Group,
+				},
+				Resources: []string{
+					"imagestreams",
+					"imagestreamtags",
+				},
+				Verbs: Verbs,
+			},
+			{
+				APIGroups: []string{
 					api.SchemeGroupVersion.Group,
 				},
-				Resources: []string{"*"},
-				Verbs:     []string{"*"},
+				Resources: []string{
+					"kieapps",
+					"kieapps/finalizers",
+				},
+				Verbs: Verbs,
 			},
 			{
 				APIGroups: []string{
@@ -191,127 +273,43 @@ func GetRole(operatorName string) *rbacv1.Role {
 					csvv1.SchemeGroupVersion.Group,
 				},
 				Resources: []string{"clusterserviceversions"},
-				Verbs:     []string{"*"},
+				Verbs: []string{
+					"get",
+					"list",
+					"patch",
+					"update",
+					"watch",
+				},
 			},
-			{
-				APIGroups: []string{
-					appsv1.SchemeGroupVersion.Group,
-				},
-				ResourceNames: []string{operatorName},
-				Resources:     []string{"deployments/finalizers"},
-				Verbs:         []string{"update"},
-			},
-			// swap out once 7.5.1 roles off, gets rid of wildcard perms
-			/*
-				{
-					APIGroups: []string{
-						"",
-					},
-					Resources: []string{
-						"configmaps",
-						"pods",
-						"services",
-						"serviceaccounts",
-						"persistentvolumeclaims",
-						"secrets",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						appsv1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"deployments",
-						"deployments/finalizers",
-						"replicasets",
-						"statefulsets",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						oappsv1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"deploymentconfigs",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						rbacv1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"rolebindings",
-						"roles",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						routev1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"routes",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						buildv1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"buildconfigs",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						oimagev1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"imagestreams",
-						"imagestreamtags",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						api.SchemeGroupVersion.Group,
-					},
-					Resources: []string{
-						"kieapps",
-						"kieapps/finalizers",
-					},
-					Verbs: Verbs,
-				},
-				{
-					APIGroups: []string{
-						monv1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{"servicemonitors"},
-					Verbs:     []string{"get", "create"},
-				},
-				{
-					APIGroups: []string{
-						csvv1.SchemeGroupVersion.Group,
-					},
-					Resources: []string{"clusterserviceversions"},
-					Verbs: []string{
-						"get",
-						"list",
-						"patch",
-						"update",
-						"watch",
-					},
-				},
-			*/
 		},
 	}
 	return role
 }
 
+func GetClusterRole(operatorName string) *rbacv1.ClusterRole {
+	return &rbacv1.ClusterRole{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: rbacv1.SchemeGroupVersion.String(),
+			Kind:       "ClusterRole",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name: operatorName,
+		},
+		Rules: []rbacv1.PolicyRule{
+			{
+				APIGroups: []string{consolev1.GroupVersion.Group},
+				Resources: []string{"consolelinks", "consoleyamlsamples"},
+				Verbs: []string{
+					"get",
+					"create",
+					"update",
+					"delete",
+				},
+			},
+		},
+	}
+}
+
 func GetCrd() *extv1beta1.CustomResourceDefinition {
 	plural := "kieapps"
 	crd := &extv1beta1.CustomResourceDefinition{
@@ -323,9 +321,8 @@ func GetCrd() *extv1beta1.CustomResourceDefinition {
 			Name: plural + "." + api.SchemeGroupVersion.Group,
 		},
 		Spec: extv1beta1.CustomResourceDefinitionSpec{
-			Scope:   "Namespaced",
-			Group:   api.SchemeGroupVersion.Group,
-			Version: api.SchemeGroupVersion.Version,
+			Scope: "Namespaced",
+			Group: api.SchemeGroupVersion.Group,
 			Versions: []extv1beta1.CustomResourceDefinitionVersion{
 				{
 					Name:    api.SchemeGroupVersion.Version,
